@@ -1,5 +1,5 @@
 package Test::PostgreSQL;
-use 5.14.0;
+use 5.014;
 use strict;
 use warnings;
 use Moo;
@@ -13,7 +13,7 @@ use File::Which;
 use POSIX qw(SIGQUIT SIGKILL WNOHANG getuid setuid);
 use User::pwent;
 
-our $VERSION = '1.26';
+our $VERSION = '1.28';
 our $errstr;
 
 # Deprecate use of %Defaults as we want to remove this package global
@@ -147,6 +147,25 @@ has unix_socket => (
   default => 0,
 );
 
+has pg_version => (
+    is => 'ro',
+    isa => Str,
+    lazy => 1,
+    predicate => 1,
+    builder => "_pg_version_builder",
+);
+
+method _pg_version_builder() {
+    my $ver_cmd = join ' ', (
+        $self->postmaster,
+        '--version'
+    );
+    
+    my ($ver) = qx{$ver_cmd} =~ /(\d+(?:\.\d+)?)/;
+    
+    return $ver;
+}
+
 has pg_ctl => (
   is => "ro",
   isa => Maybe[Str],
@@ -159,7 +178,7 @@ method _pg_ctl_builder() {
   if ( $prog ) {
       # we only use pg_ctl if Pg version is >= 9
       my $ret = qx/"$prog" --version/;
-      if ( $ret =~ /(\d+)(?:\.|devel)/ && $1 >= 9 ) {
+      if ( $ret =~ /(\d+)(?:\.|devel|beta)/ && $1 >= 9 ) {
           return $prog;
       }
       warn "pg_ctl version earlier than 9";
@@ -201,10 +220,24 @@ has extra_psql_args => (
 has run_psql_args => (
     is => 'ro',
     isa => Str,
-    # Single transaction, skip .psqlrc, be quiet, echo errors, stop on first error
-    default => '-1Xqb -v ON_ERROR_STOP=1',
+    lazy => 1,
+    builder => "_build_run_psql_args",
 );
 
+method _build_run_psql_args() {
+    my @args = (
+        '-1', # Single transaction
+        '-X', # Ignore .psqlrc
+        '-q', # Quiet
+        '-v ON_ERROR_STOP=1', # Stop on first error
+    );
+    
+    # Echo errors, available in psql 9.5+
+    push @args, '-b' if $self->pg_version >= 9.5;
+    
+    return join ' ', @args;
+}
+
 has seed_scripts => (
     is => 'ro',
     isa => ArrayRef[Str],
@@ -357,7 +390,8 @@ method start() {
 method _find_port_and_launch() {
   my $tries = 10;
   my $port = $self->base_port;
-  # try by incrementing port number
+  srand(); # Re-seed the RNG in case the caller forked the process
+  # try by incrementing port number until PostgreSQL starts
   while (1) {
     my $good = try {
       $self->_try_start($port);
@@ -366,12 +400,14 @@ method _find_port_and_launch() {
     catch {
       # warn "Postgres failed to start on port $port\n";
       unless ($tries--) {
-        die "Failed to start postgres on port $port: $_";
+        die "Failed to start postgres after trying 10 potential ports: $_";
       }
       undef;
     };
     return if $good;
-    $port++;
+    # Increment port by a random number to avoid clashes with other Test::Postgresql processes
+    # Keep in mind that this increment is going to be made up to 10 times, so avoid exceeding 64k
+    $port += int(rand(500)) + 1;
   }
 }
 
@@ -664,9 +700,19 @@ method run_psql(@psql_args) {
 }
 
 method run_psql_scripts(@script_paths) {
-    my $psql_args = join ' ', map {; "-f $_" } @script_paths;
+    my @psql_commands;
     
-    $self->run_psql($psql_args);
+    # psql 9.6+ supports multiple -c and -f commands invoked at once,
+    # older psql does not. Executing psql multiple times breaks single
+    # transaction semantics but is unlikely to cause problems in real world.
+    if ( $self->pg_version > 9.6 ) {
+        push @psql_commands, join ' ', map {; "-f $_" } @script_paths;
+    }
+    else {
+        @psql_commands = map {; "-f $_" } @script_paths;
+    }
+    
+    $self->run_psql($_) for @psql_commands;
 }
 
 1;
@@ -771,7 +817,7 @@ to try and find it in PostgreSQL directory.
 Arguments to pass to C<initdb> program when creating a new PostgreSQL database
 cluster for Test::PostgreSQL session.
 
-Defaults to C<-U postgres -A trust>. See L</db_owner>.
+Defaults to C<-U postgres -A trust>. See L</dbowner>.
 
 =head2 extra_initdb_args
 
@@ -831,7 +877,7 @@ instance.
 
 Defaults to C<-U postgres -d test -h 127.0.0.1 -p $self-E<gt>port>.
 
-See also L</db_owner>, L</dbname>, L</host>, L</base_port>.
+See also L</dbowner>, L</dbname>, L</host>, L</base_port>.
 
 =head2 extra_psql_args
 
@@ -860,7 +906,7 @@ q: Run quietly, print only notices and errors on stderr (if any)
 
 =item *
 
-b: Echo SQL statements that cause PostgreSQL exceptions
+b: Echo SQL statements that cause PostgreSQL exceptions (version 9.5+)
 
 =item *
 
@@ -873,6 +919,11 @@ b: Echo SQL statements that cause PostgreSQL exceptions
 Arrayref with the list of SQL scripts to run after the database was instanced
 and set up. Default is C<[]>.
 
+B<NOTE> that C<psql> older than 9.6 does not support multiple C<-c> and C<-f>
+switches in arguments so C<seed_scripts> will be executed one by one. This
+implies multiple transactions instead of just one; if you need all seed statements
+to apply within a single transaction, combine them into one seed script.
+
 =head2 auto_start
 
 Integer value that controls whether PostgreSQL server is started and setup
@@ -956,10 +1007,14 @@ to escape them manually like shown above. C<run_psql> will not quote them for yo
 The actual command line to execute C<psql> will be concatenated from L</psql_args>,
 L</extra_psql_args>, and L</run_psql_args>.
 
+B<NOTE> that C<psql> older than 9.6 does not support multiple C<-c> and/or C<-f>
+switches in arguments.
+
 =head2 run_psql_scripts
 
 Given a list of script file paths, invoke L</run_psql> once with C<-f 'script'>
-for every path.
+for every path in PostgreSQL 9.6+, or once per C<-f 'script'> for older PostgreSQL
+versions.
 
 =head1 ENVIRONMENT
 
